/*Author: Andrew Kuklin
Requester: Ian Kirby
Date: 30 May 2001
Description: Text strategy as a project for the course of Artificial Intelligence
*/


/* Comments:
	1. I assume that farmers produce farmers, labourers - labourers, and soldiers - soldiers.
	2. Rate of reproduction is halved for labourers, and divided by 4 for soldiers, so if
		farmers reproduce with rate of 1.05 then corresponding figure for labourers and 
		soldiers will be 1.025 and 1.0125.
	3. Farmers eat 0.3 units of food per turn, labourers - 0.45, soldiers - 0.6
	4. I didn't place any factors before (OldFood-ReqFood) in eat() predicate because I assumed 
		that not all people die of famine. So if we have 0 food, 100 soldiers and no farmers
		or labourers only 0.6*100=60 soldiers will die.
	5. For the first time I assume that walls are not destructed.
	6. During plundering there are no losses from any side, and attackers steal half of food and
		resourses of defender.
	7. I assume that computer has perfect information about human situation, and doesn't need spying. 
*/

/* Bugs:
	?Corrected1. Bug with equal army strength (all attackers are killed).
	2. Computer makes soldiers only of farmers.
	3. Computer doesn't create soldiers for defence.
	4. Wrong calculation during battle .
	5. Bug in famine (killing non-existent farmers).
	6. Big Bug. Strange calculations of strength, just walls can defend.
	8. Strategy drawback in Mr. Kirby's project. You can just develop providing some 
		defenders and computer will finally starve his population unnecessarily
		defending.		 
*/	
constants
	sTARTINGFOOD      = 40
	sTARTINGRESOURCES = 20
	sTARTFARMERS	  = 90
	sTARTLABOURERS    = 10
	sTARTSOLDIERS     = 0
	sTARTWALL	  = 0
	
	pOPULATIONGROWTH  = 1.05
	sTARTFOODFACTOR   = 1.2
	sTARTRESOURCEFACTOR = 1.0
	sTARTFIGHTFACTOR  = 1.0
	
% 	Food for 1 farmer
	eATFACTOR=0.3 
	
	sTARTYEAR         = 687
	lASTYEAR	  = 750
	
	cOSTTOMAKESOLDIER = 0.5

%	Technologies upgrade
	fIGHTFACTORBOOST  = 0.05
	

database
	currentYear(integer theYear)
	
	foodStatus(symbol thePlayer, integer theFood)
	resourceStatus(symbol thePlayer, integer theResources)
	peopleStatus(symbol thePlayer, integer theFarmers, 
				integer theLabourers, integer theSoldiers)
	army(symbol thePlayer, integer attackers, integer defenders),
	wallStatus(symbol thePlayer, integer theWall)
	reproductionFactorStatus(symbol thePlayer, real repFactor)
	foodFactorStatus(symbol thePlayer, real foodFacotr)
	resourceFactorStatus(symbol thePlayer, real resFactor)
	fightFactorStatus(symbol thePlayer, real fightFactor)


	%%%%%%%%%%%%%%%%%%%%%%%%%%% SETUPPLAYERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

predicates
	setupPlayers	          % give starting values to both computer and human
	setup(symbol whom)
	
clauses
	setup(Whom) :-	
		assert(foodStatus(Whom, sTARTINGFOOD)),
		assert(resourceStatus(Whom, sTARTINGRESOURCES)),
		assert(peopleStatus(Whom, sTARTFARMERS, sTARTLABOURERS, sTARTSOLDIERS)),
		assert(wallStatus(Whom, sTARTWALL)),
		assert(foodFactorStatus(Whom, sTARTFOODFACTOR)),
		assert(reproductionFactorStatus(Whom, pOPULATIONGROWTH)),
		assert(resourceFactorStatus(Whom, sTARTRESOURCEFACTOR)),
		assert(fightFactorStatus(Whom, sTARTFIGHTFACTOR)),
		assert(army(computer, 0, 6)). % If human attack at first turn

	setupPlayers :-
		setup(human),
		setup(computer),
		assert(currentYear(sTARTYEAR)).


	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/DISPLAY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

predicates
	nondeterm displaySituation
	nondeterm displayWall(integer wallStrength)
	nondeterm displayTechnologies(real repFactor, real foodFactor, real resFactor, real fightFactor)
	
clauses
	displayWall(0) :-
		write("You have no walls to protect you.\n\n").
		
	displayWall(W) :-
		W > 0,
		write("Your walls are currently of strength: ", W, "\n\n").
		
	displayTechnologies(RepFactor, FoodFactor, ResFactor, FightFactor) :-
		write("Technologies\n"),
		write("    Reproduction rate:\t", RepFactor, "\n"), 
		write("    Food Production:\t\t", FoodFactor, "\n"),
		write("    Resource Production:\t", ResFactor, "\n"),
		write("    Military Technology:\t", FightFactor, "\n\n").
	
	displaySituation :-
		currentYear(Year),
		write("\n\n**************** ", Year, "****************\n\n"),
		write("It is the year ", Year, " and here's how you're doing.\n\n"), 
		foodStatus(human, Food),
		resourceStatus(human, Res),
		peopleStatus(human, F, L, S),
		wallStatus(human, W),
		reproductionFactorStatus(human, RepFactor),
		foodFactorStatus(human, FoodFactor),
		resourceFactorStatus(human, ResFactor),
		fightFactorStatus(human, FightFactor),
		
		Pop = F + L + S,
		write("You have a population of ", Pop), nl,
		write("    soldiers:\t", S, "\n"),
		write("    labourers:\t", L, "\n"),
		write("    farmers:\t", F, "\n\n"),
		write("Your supplies:\n"),
		write("    food:\t\t", Food, "\n"),
		write("    resources:\t", Res, "\n\n"),
		
		displayWall(W),
		displayTechnologies(RepFactor, FoodFactor, ResFactor, FightFactor).	

	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/GETCOMMANDS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

predicates
	nondeterm getCommands
	nondeterm handleSoldierNumber(integer desiredNewNum, integer oldNum, integer totalPopulation, 
			integer actualNewNum)
	nondeterm subFoodForSoldiers(integer numOfDesiredSoldiers, integer oldNumofSoldiers, 
			integer totalPop, integer actualNewNumSoldiers)
	nondeterm handleAttackersAndDefenders(integer totNumSoldiers, integer desiredNumAttackers)
	nondeterm handleLabourerNumber(integer desiredLabs, integer numSoldiers, integer totalPopulation, 
			integer definiteLabs)
	nondeterm handleWallAdd(integer desiredAdd, integer successfullAdd)
	nondeterm checkNumberofSoldiers (integer army, integer attackers)
	
	nondeterm produce(symbol who)
	nondeterm eat(symbol who, real food)
	nondeterm researchReproduction(symbol who)
	nondeterm researchFood(symbol who)
	nondeterm researchTechnology(symbol who)
	nondeterm researchFight(symbol who)
	nondeterm askForSpy
	nondeterm showSpyInfo(integer resourcesSpent, real deviation)
	nondeterm createDeviation (integer resourcesSpent, real deviation)
	nondeterm checkForResources (integer resources)
	nondeterm getDesiredSoldiers
	nondeterm getDesiredLabourers
	nondeterm getDesiredWall
clauses
%=============Handling number of soldiers, attackers/defenders, walls, labourers and farmers==============
	getDesiredSoldiers :- 
		peopleStatus(human, OldF, OldL, OldS),
		Pop = OldF + OldL + OldS,
		write("Assign your population:\n"),
		write("    soldiers: "),
		readint(S),
		handleSoldierNumber(S, OldS, Pop, NewS),		
		checkNumberofSoldiers(NewS, A),
		handleAttackersAndDefenders(NewS, A),
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, OldF, OldL, NewS)).
		
	getDesiredLabourers :-
		peopleStatus(human, OldF, OldL, OldS),
		Pop = OldF + OldL + OldS,
		write("How many people should work on your resources?\n"),
		write("    labourers: "),
		readint(L),
		handleLabourerNumber(L, OldS, Pop, NewL),
		NewF = Pop - OldS - NewL,
		write("Therefore you have\n"),
		write("    farmers: ", NewF, "\n\n"),
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, NewF, NewL, OldS)).
		
	getDesiredWall :-
		write("How much do you want to strengthen your walls? (costs 1 unit of resources per unit of wall)\n"),
		write("    walls: "),
		readint(DesiredW),
		handleWallAdd(DesiredW, ActualW),
		retract(wallStatus(human, PrevW)),
		NewW = PrevW + ActualW,
		assert(wallStatus(human, NewW)).
		
	subFoodForSoldiers(DesiredSoldiers, OldS,  _, DesiredSoldiers) :- 
		DesiredSoldiers <= OldS.			

	subFoodForSoldiers(DesiredSoldiers, OldS, Pop, NewSoldiers) :-
		DesiredSoldiers > OldS,
		Change = DesiredSoldiers - OldS,
		foodStatus(human, F),
		F < (Change div 2),
		write("It takes some bread to make a peasant into a soldier.\n"),
		write("Try a lower number.\n"),
		write("    soldiers: "),
		readint(S),
		handleSoldierNumber(S, OldS, Pop, NewSoldiers).

	subFoodForSoldiers(DesiredSoldiers, OldS,  _, DesiredSoldiers) :-
		DesiredSoldiers > OldS,
		Change = DesiredSoldiers - OldS,
		foodStatus(human, F),
		F >= (Change div 2),
		retract(foodStatus(human, F)),
		NewF = F - ((DesiredSoldiers + 1) div 2),
		assert(foodStatus(human, NewF)).
		
	handleSoldierNumber(DesiredS, OldS, Pop, DefiniteS) :-
		DesiredS > Pop,
		write("You don't have that many people.\n"),
		write("    soldiers: "),
		readint(S),
		handleSoldierNumber(S, OldS, Pop, DefiniteS).

	handleSoldierNumber(DesiredS, OldS, Pop, NewS) :-
		DesiredS <= Pop,
		subFoodForSoldiers(DesiredS, OldS, Pop, NewS).

	handleAttackersAndDefenders(TotalSoldiers, ToAttack) :-
		ToAttack > TotalSoldiers,
		write("You don't have so many soldiers.\n"),
		write("    to attack: "),
		readint(A),
		handleAttackersAndDefenders(TotalSoldiers, A).

	handleAttackersAndDefenders(TotalSoldiers, ToAttack) :-
		ToAttack <= TotalSoldiers,
		retractall(army(human,  _,  _)),
		ToDef = TotalSoldiers - ToAttack,
		write("Assigning ", ToDef, " soldiers to defend your society.\n"),
		assert(army(human, ToAttack, ToDef)).
		
	checkNumberofSoldiers (Army, Attackers) :-
		Army=0,
		write("Assigning 0 attackers to advance guard. \n"),
		Attackers=0.

	checkNumberofSoldiers (Army, Attackers) :-
		Army<>0,
		write("How many of these soldiers should attack?\n"),
		write("    to attack: "),
		readint(Attackers).
			
	handleLabourerNumber(L, S, Pop, DefiniteL) :-
		L > (Pop - S),
		write("You can't have so many labourers.\n"),
		write("    labourers: "),
		readint(NewL),
		handleLabourerNumber(NewL, S, Pop, DefiniteL).

	handleLabourerNumber(L, S, Pop, L) :-
		L <= (Pop - S).
		
	handleWallAdd(DesiredW, NewW) :-
		resourceStatus(human, R),
		R < DesiredW,
		write("You don't have enough resources to build that big a wall.\n"),
		write("    wall: "),
		readint(W),
		handleWallAdd(W, NewW).

	handleWallAdd(DesiredW, DesiredW) :-
		resourceStatus(human, R),
		R >= DesiredW,
		retract(resourceStatus(human, R)),
		NewR = R - DesiredW,
		assert(resourceStatus(human, NewR)).

%==============Eat and die of famine====================
% Enough food		
	eat(Who, ReqFood) :-
		Who=human,
		foodStatus(human, OldFood),
		OldFood>ReqFood,
		NewFood=val(integer, OldFood-ReqFood),
		retract(foodStatus(human, OldFood)),
		assert(foodStatus(human, NewFood)).

% Killing farmers is enough	
	eat(Who, ReqFood) :-
		Who=human,
		foodStatus(human, OldFood),
		peopleStatus(human, OldF, OldL, OldS),
		OldFood<ReqFood,
		NewF=val(integer, OldF-(ReqFood-OldFood)),
		NewF>0,
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, NewF, OldL, OldS)),
		retract(foodStatus(human, OldFood)),
		assert(foodStatus(human, 0)),
		DiedF=OldF-NewF,
		write(DiedF, " farmers have died of famine!").

% Killing farmers is not enough but extra labourers will do
	eat(Who, ReqFood) :-
		Who=human,
		foodStatus(human, OldFood),
		peopleStatus(human, OldF, OldL, OldS),
		OldF+OldFood-ReqFood<0,
		OldL+OldF+OldFood-ReqFood>0,		
		NewL=val(integer, OldL+OldF+OldFood-ReqFood),
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, 0, NewL, OldS)),
		retract(foodStatus(human, OldFood)),
		assert(foodStatus(human, 0)),
		Temp=OldL-NewL,
		write(OldF, " farmers and ", Temp, " labourers have died of famine!").		
		
% Kill farmers and labourers is not enough but soldiers are saving the country
	eat(Who, ReqFood) :-
		Who=human,
		foodStatus(human, OldFood),
		peopleStatus(human, OldF, OldL, OldS),
		OldL+OldF+OldFood-ReqFood<0,		
		OldS+OldL+OldF+OldFood-ReqFood>0,		
		NewS=val(integer, OldS+OldL+OldF+OldFood-ReqFood),
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, 0, 0, NewS)),
		retract(foodStatus(human, OldFood)),
		assert(foodStatus(human, 0)),
		Temp=OldS-NewS,
		write(OldF, " farmers, ", OldL, " labourers and ", Temp, " soldiers have died of famine!").		
		
% Kill 'em all!
	eat(Who, ReqFood) :-
		Who=human,
		foodStatus(human, OldFood),
		peopleStatus(human, OldF, OldL, OldS),
		OldS+OldL+OldF+OldFood-ReqFood<0,		
		retract(peopleStatus(human, OldF, OldL, OldS)),
		assert(peopleStatus(human, 0, 0, 0)),
		retract(foodStatus(human, OldFood)),
		assert(foodStatus(human, 0)).		

%=======Everything grows==========
	produce(Who) :-
		Who=human,
		retract(resourceStatus(human, OldRes)),
		retract(peopleStatus(human, OldF, OldL, OldS)),
		retract(foodStatus(human, OldFood)),
		reproductionFactorStatus(human, RepFactor),
		foodFactorStatus(human, FoodFactor),
		resourceFactorStatus(human, ResFactor),
		NewRes=val(integer, OldRes+ResFactor*OldL),
		NewF=val(integer, OldF*RepFactor),
		NewL=val(integer, OldL*((RepFactor-1)/2+1)),
		NewS=val(integer, OldS*((RepFactor-1)/4+1)),
		NewFood=val(integer, OldFood+FoodFactor*OldF),
		assert(resourceStatus(human, NewRes)),
		assert(peopleStatus(human, NewF, NewL, NewS)),
		assert(foodStatus(human, NewFood)).
				
	researchFight(Who) :-
		Who=human,
		write("Enter the level of fight upgrade (each level costs 1 resource)"),
		readint(Level),
		resourceStatus(human, OldRes),
		Level<=OldRes,
		retract(resourceStatus(human, OldRes)),
		NewRes=OldRes-Level,
		assert(resourceStatus(human, NewRes)),
		retract(fightFactorStatus(human, Factor)),
		NewFactor=Factor+fIGHTFACTORBOOST*Level,
		assert(fightFactorStatus(human, NewFactor)).

%========Interact with user and call above predicates==========				
	getCommands :-
		askForSpy,
		getDesiredSoldiers,
		getDesiredLabourers,
		getDesiredWall,
		researchReproduction(human),
		researchFood(human),
		researchTechnology(human),
		researchFight(human),
		peopleStatus(human, F, L, S),
		Food=F*eATFACTOR+L*eATFACTOR*1.5+S*eATFACTOR*2,
		eat(human, Food),
		produce(human),
		write("\n\nContinue..."),
		readln(_).

	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/MAKE COMPUTER DECISIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

predicates
	nondeterm makeComputerDecisions
	nondeterm compDefend
	nondeterm compAttack
	nondeterm compDevelop
	nondeterm compProduce(symbol who)
	nondeterm compEat(symbol who, real food)
	
clauses
	makeComputerDecisions :-
		compDefend,
		compAttack,
		compDevelop.
		
%	Current defence is ok
	compDefend :-
		army(computer, _, CompDefenders),
		army(human, HumAttackers, _),		
		wallStatus(computer, CompWall),
		fightFactorStatus(computer, CompFightFactor),
		fightFactorStatus(human, HumFightFactor),
		(CompDefenders+CompWall)*CompFightFactor >= HumAttackers*HumFightFactor.

%	Building some walls is enough for repelling stupid humans and there are enough resources for building them
	compDefend :-
		army(computer, _, CompDefenders),
		army(human, HumAttackers, _),		
		wallStatus(computer, CompWall),
		fightFactorStatus(computer, CompFightFactor),
		fightFactorStatus(human, HumFightFactor),
		resourceStatus(computer, CompRes),
		(CompDefenders+CompWall)*CompFightFactor < HumAttackers*HumFightFactor,
		AddedWall=(HumAttackers*HumFightFactor)/CompFightFactor-CompDefenders-CompWall,
		AddedWall<CompRes,
		NewWall=CompWall+AddedWall,
		NewRes=CompRes-AddedWall,
		retract(resourceStatus(computer, CompRes)),			
		assert(resourceStatus(computer, NewRes)),
		retract(wallStatus(computer, CompWall)),
		assert(wallStatus(computer, NewWall)).
		
%	Building walls is not enough, build walls for half of resources, another half will go for increasing fight factor, 
%		and this will be enough
	compDefend :-
		army(computer, _, CompDefenders),
		army(human, HumAttackers, _),		
		wallStatus(computer, CompWall),
		fightFactorStatus(computer, CompFightFactor),
		fightFactorStatus(human, HumFightFactor),
		resourceStatus(computer, CompRes),
		(CompDefenders+CompWall)*CompFightFactor < HumAttackers*HumFightFactor,
		AddedWall=(HumAttackers*HumFightFactor)/CompFightFactor-CompDefenders-CompWall,
		AddedWall>CompRes,
		NewWall=CompWall+(CompRes div 2),
		NewCompFightFactor=CompFightFactor+(CompRes div 2)*fIGHTFACTORBOOST,
		(CompDefenders+CompWall+AddedWall)*NewCompFightFactor >= HumAttackers*HumFightFactor,		
		retract(resourceStatus(computer, CompRes)),			
		assert(resourceStatus(computer, 0)),
		retract(wallStatus(computer, CompWall)),
		assert(wallStatus(computer, NewWall)),
		retract(fightFactorStatus(computer, CompFightFactor)),
		assert(fightFactorStatus(computer, NewCompFightFactor)).
		
%	Neither walls, nor increase of fight factor can save computer, increase number of soldiers for all
%		available food (if food limits stricter than available farmers)
	compDefend :-
		army(computer, CompAttackers, CompDefenders),
		army(human, HumAttackers, _),		
		wallStatus(computer, CompWall),
		fightFactorStatus(computer, CompFightFactor),
		fightFactorStatus(human, HumFightFactor),
		resourceStatus(computer, CompRes),
		foodStatus(computer, CompFood),
		peopleStatus(computer, CompF, CompL, CompS),
		(CompDefenders+CompWall)*CompFightFactor < HumAttackers*HumFightFactor,
		AddedWall=(HumAttackers*HumFightFactor)/CompFightFactor-CompDefenders-CompWall,
		AddedWall>CompRes,
		NewWall=CompWall+(CompRes div 2),
		NewCompFightFactor=CompFightFactor+(CompRes div 2)*fIGHTFACTORBOOST,
		(CompDefenders+CompWall+AddedWall)*NewCompFightFactor < HumAttackers*HumFightFactor,
		AddedCompDefendersByFood=CompFood*cOSTTOMAKESOLDIER,
		AddedCompDefendersByPopulation=CompF,
		AddedCompDefendersByFood>AddedCompDefendersByPopulation,
		NewCompDefenders=CompDefenders+AddedCompDefendersByFood,
		NewF=NewCompDefenders-CompDefenders,
		retract(resourceStatus(computer, CompRes)),			
		assert(resourceStatus(computer, 0)),
		retract(wallStatus(computer, CompWall)),
		assert(wallStatus(computer, NewWall)),
		retract(fightFactorStatus(computer, CompFightFactor)),
		assert(fightFactorStatus(computer, NewCompFightFactor)),
		retract(foodStatus(computer, CompFood)),
		assert(foodStatus(computer, 0)),
		retract(army(computer, _, CompDefenders)),
		assert(army(computer, CompAttackers, NewCompDefenders)),
		retract(peopleStatus(computer, CompF, CompL, _)),
		assert(peopleStatus(computer, NewF, CompL, CompS)).
		
%	Neither walls, nor increase of fight factor can save computer, set all available population
%	to soldiers (if population limits stricter than available food)	
	compDefend :-
		army(computer, CompAttackers, CompDefenders),
		army(human, HumAttackers, _),		
		wallStatus(computer, CompWall),
		fightFactorStatus(computer, CompFightFactor),
		fightFactorStatus(human, HumFightFactor),
		resourceStatus(computer, CompRes),
		foodStatus(computer, CompFood),
		peopleStatus(computer, CompF, CompL, CompS),
		(CompDefenders+CompWall)*CompFightFactor < HumAttackers*HumFightFactor,
		AddedWall=(HumAttackers*HumFightFactor)/CompFightFactor-CompDefenders-CompWall,
		AddedWall>CompRes,
		NewWall=CompWall+(CompRes div 2),
		NewCompFightFactor=CompFightFactor+(CompRes div 2)*fIGHTFACTORBOOST,
		(CompDefenders+CompWall+AddedWall)*NewCompFightFactor < HumAttackers*HumFightFactor,
		AddedCompDefendersByFood=CompFood*cOSTTOMAKESOLDIER,
		AddedCompDefendersByPopulation=CompF,
		AddedCompDefendersByFood<AddedCompDefendersByPopulation,
		NewCompDefenders=CompDefenders+AddedCompDefendersByPopulation,
		NewFood=(NewCompDefenders-CompDefenders)*2,
		retract(resourceStatus(computer, CompRes)),			
		assert(resourceStatus(computer, 0)),
		retract(wallStatus(computer, CompWall)),
		assert(wallStatus(computer, NewWall)),
		retract(fightFactorStatus(computer, CompFightFactor)),
		assert(fightFactorStatus(computer, NewCompFightFactor)),
		retract(foodStatus(computer, CompFood)),
		assert(foodStatus(computer, NewFood)),
		retract(army(computer, _, CompDefenders)),
		assert(army(computer, CompAttackers, NewCompDefenders)),
		retract(peopleStatus(computer, CompF, CompL, _)),
		assert(peopleStatus(computer, 0, CompL, CompS)).				

%	Don't attack if computer has not enough population to break through 
%	(more than 30% of farmers or labourers needed for adequate increase)
	compAttack :-
		peopleStatus(computer, CompF, _, CompS),
		army(human, _, HumDefenders),
		wallStatus(human, HumWall),
		fightFactorStatus(human, HumFF),
		fightFactorStatus(computer, CompFF),
		ExtraUnits=(HumDefenders+HumWall)*HumFF/CompFF-CompS,
		ExtraUnits>0.3*(CompF+CompS).
		
%	Don't attack if you don't have enough food
%	(more than 60% of food supplies needed to squash mere humans)
	compAttack :-
		peopleStatus(computer, _, _, CompS),
		army(human, _, HumDefenders),
		wallStatus(human, HumWall),
		fightFactorStatus(human, HumFF),
		fightFactorStatus(computer, CompFF),
		foodStatus(computer, CompFood),
		ExtraUnits=(HumDefenders+HumWall)*HumFF/CompFF-CompS,
		ExtraUnits>CompFood*0.6/2.
		
%	Attack if there are no defenders
	compAttack :-
		peopleStatus(computer, _, _, CompS),
		army(human, _, HumDefenders),
		wallStatus(human, HumWall),
		fightFactorStatus(human, HumFF),
		fightFactorStatus(computer, CompFF),
		foodStatus(computer, CompFood),
		ExtraUnits=(HumDefenders+HumWall)*HumFF/CompFF-CompS,
		ExtraUnits>CompFood*0.6/2.	
			
	
	compDevelop.	
		
%==============Eat and die of famine for computer====================
% Enough food		
	compEat(Who, ReqFood) :-
		Who=computer,
		foodStatus(computer, OldFood),
		OldFood>ReqFood,
		NewFood=val(integer, OldFood-ReqFood),
		retract(foodStatus(computer, OldFood)),
		assert(foodStatus(computer, NewFood)).

% Killing farmers is enough	
	compEat(Who, ReqFood) :-
		Who=computer,
		foodStatus(computer, OldFood),
		peopleStatus(computer, OldF, OldL, OldS),
		OldFood<ReqFood,
		NewF=val(integer, OldF-(ReqFood-OldFood)),
		NewF>0,
		retract(peopleStatus(computer, OldF, OldL, OldS)),
		assert(peopleStatus(computer, NewF, OldL, OldS)),
		retract(foodStatus(computer, OldFood)),
		assert(foodStatus(computer, 0)),
		write("Spies report about slight famine in computer country.").

% Killing farmers is not enough but extra labourers will do
	compEat(Who, ReqFood) :-
		Who=computer,
		foodStatus(computer, OldFood),
		peopleStatus(computer, OldF, OldL, OldS),
		OldF+OldFood-ReqFood<0,
		OldL+OldF+OldFood-ReqFood>0,		
		NewL=val(integer, OldL+OldF+OldFood-ReqFood),
		retract(peopleStatus(computer, OldF, OldL, OldS)),
		assert(peopleStatus(computer, 0, NewL, OldS)),
		retract(foodStatus(computer, OldFood)),
		assert(foodStatus(computer, 0)),
		write("Spies report about substantial famine in computer country.").		
		
% Kill farmers and labourers is not enough but soldiers are saving the country
	compEat(Who, ReqFood) :-
		Who=computer,
		foodStatus(computer, OldFood),
		peopleStatus(computer, OldF, OldL, OldS),
		OldL+OldF+OldFood-ReqFood<0,		
		OldS+OldL+OldF+OldFood-ReqFood>0,		
		NewS=val(integer, OldS+OldL+OldF+OldFood-ReqFood),
		retract(peopleStatus(computer, OldF, OldL, OldS)),
		assert(peopleStatus(computer, 0, 0, NewS)),
		retract(foodStatus(computer, OldFood)),
		assert(foodStatus(computer, 0)),
		write("Spies report about big famine in computer country.").		
		
% Kill 'em all!
	compEat(Who, ReqFood) :-
		Who=computer,
		foodStatus(computer, OldFood),
		peopleStatus(computer, OldF, OldL, OldS),
		OldS+OldL+OldF+OldFood-ReqFood<0,		
		retract(peopleStatus(computer, OldF, OldL, OldS)),
		assert(peopleStatus(computer, 0, 0, 0)),
		retract(foodStatus(computer, OldFood)),
		assert(foodStatus(computer, 0)).		

%=======Everything grows==========
	compProduce(Who) :-
		Who=computer,
		retract(resourceStatus(computer, OldRes)),
		retract(peopleStatus(computer, OldF, OldL, OldS)),
		retract(foodStatus(computer, OldFood)),
		reproductionFactorStatus(computer, RepFactor),
		foodFactorStatus(computer, FoodFactor),
		resourceFactorStatus(computer, ResFactor),
		NewRes=val(integer, OldRes+ResFactor*OldL),
		NewF=val(integer, OldF*RepFactor),
		NewL=val(integer, OldL*((RepFactor-1)/2+1)),
		NewS=val(integer, OldS*((RepFactor-1)/4+1)),
		NewFood=val(integer, OldFood+FoodFactor*OldF),
		assert(resourceStatus(computer, NewRes)),
		assert(peopleStatus(computer, NewF, NewL, NewS)),
		assert(foodStatus(computer, NewFood)).		
	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/FIGHT WARS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	

	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/FIGHT WARS/HUMAN 
predicates
	nondeterm fightHumanWar.
	nondeterm robOrFight(integer choice, integer soldaten).
	
clauses
%	No war
	fightHumanWar :-
		army(human, 0, _).
	
%	Normal war, human wins
	fightHumanWar :-
		army(human, HumAttackers, HumDefenders),
		army(computer, CompAttackers, CompDefenders),
		wallStatus(computer, CompWall),
		fightFactorStatus(human, HumFightFactor),
		fightFactorStatus(computer, CompFightFactor),
		HumAttackers*HumFightFactor > (CompDefenders+CompWall)*CompFightFactor,
		NewHumAttackers=val(integer, (HumFightFactor*HumAttackers-CompFightFactor*CompDefenders)/(HumFightFactor*HumAttackers)*(1+
					(HumFightFactor*HumAttackers-CompFightFactor*CompDefenders)/(HumFightFactor*HumAttackers))*HumAttackers),
		retract(army(computer, CompAttackers, CompDefenders)),
		assert(army(computer, CompAttackers, 0)),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, NewHumAttackers, HumDefenders)),
		retract(peopleStatus(human, HumF, HumL, _)),
		NewHumS=NewHumAttackers+HumDefenders,
		assert(peopleStatus(human, HumF, HumL, NewHumS)),
		retract(peopleStatus(computer, CompF, CompL, _)),		
		assert(peopleStatus(computer, CompF, CompL, CompAttackers)),
		write("You have killed all defenders and there are still ", NewHumAttackers, " men able to attack."),
		write("\n Do you want to plunder resources and food or continue attacking farmers and labourers? (1 or 2)"),
		readint(Choice),
		robOrFight(Choice, NewHumAttackers).
		
		
%	Normal war, computer wins
	fightHumanWar :-
		army(human, HumAttackers, HumDefenders),
		army(computer, CompAttackers, CompDefenders),
		wallStatus(computer, CompWall),
		fightFactorStatus(human, HumFightFactor),
		fightFactorStatus(computer, CompFightFactor),
		(CompDefenders+CompWall)*CompFightFactor >= HumAttackers*HumFightFactor,
		NewCompDefenders=val(integer, (CompFightFactor*CompDefenders-HumFightFactor*HumAttackers)/(CompFightFactor*CompDefenders)*(1+
					(CompFightFactor*CompDefenders-HumFightFactor*HumAttackers)/(CompFightFactor*CompDefenders))*CompDefenders),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, 0, HumDefenders)),
		retract(army(computer, CompAttackers, CompDefenders)),
		assert(army(computer, CompAttackers, NewCompDefenders)),
		retract(peopleStatus(computer, CompF, CompL, _)),
		NewCompS=NewCompDefenders+CompDefenders,
		assert(peopleStatus(computer, CompF, CompL, NewCompS)),
		retract(peopleStatus(human, HumF, HumL, _)),		
		assert(peopleStatus(human, HumF, HumL, HumDefenders)),
		KilledDefenders=CompDefenders-NewCompDefenders,
		write("Your attackers killed ", KilledDefenders, " and all perished").
		
%	Pillage poor farmers & labourers. Take half of food & resources.
	robOrFight(Choice, _) :-
		Choice=1,
		foodStatus(computer, CompFood),
		resourceStatus(computer, CompRes),
		foodStatus(human, HumFood),
		resourceStatus(human, HumRes),
		StolenFood=val(integer, CompFood div 2),
		StolenRes=val(integer, CompRes div 2),
		NewFood=HumFood+StolenFood,
		NewRes=HumRes+StolenRes,
		retract(foodStatus(computer, CompFood)),
		retract(resourceStatus(computer, CompRes)),
		assert(foodStatus(computer, StolenFood)),
		assert(resourceStatus(computer, StolenRes)),		
		retract(foodStatus(human, HumFood)),
		retract(resourceStatus(human, HumRes)),
		assert(foodStatus(human, NewFood)),
		assert(resourceStatus(human, NewRes)),
		write("You have stolen ", StolenFood, " units of food and ", StolenRes, " units of resourses").
		
%	Computer farmers are able to handle with attackers
	robOrFight(Choice, Soldaten) :-
		Choice=2,
		peopleStatus(computer, CompF, CompL, CompS),
		fightFactorStatus(human, FightFactor),
		army(human, HumAttackers, HumDefenders),
		CompF*0.8>Soldaten*FightFactor,
		NewF=val(integer, CompF-(Soldaten/CompF*(CompF-Soldaten))),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, 0, HumDefenders)),
		retract(peopleStatus(human, HumF, HumL, _)),
		assert(peopleStatus(human, HumF, HumL, HumDefenders)),
		retract(peopleStatus(computer, CompF, CompL, CompS)),
		assert(peopleStatus(computer, NewF, CompL, CompS)),
		KilledF=CompF-NewF,
		write("Brave farmers defended their city. You killed only ", KilledF, " of them").
		
%	Farmers and labourers together are able to defend the society
	robOrFight(Choice, Soldaten) :-
		Choice=2,
		peopleStatus(computer, CompF, CompL, CompS),
		fightFactorStatus(human, FightFactor),
		army(human, HumAttackers, HumDefenders),
		CompF*0.8 < Soldaten*FightFactor,
		CompStrength=(CompF+CompL)*0.8, 
		CompStrength > Soldaten*FightFactor,
		NewL=val(integer, CompL-(Soldaten/CompL*(CompL-Soldaten))),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, 0, HumDefenders)),
		retract(peopleStatus(human, HumF, HumL, _)),
		assert(peopleStatus(human, HumF, HumL, HumDefenders)),
		retract(peopleStatus(computer, CompF, CompL, CompS)),
		assert(peopleStatus(computer, 0, NewL, CompS)),
		KilledL=CompL-NewL,
		write("Brave farmers and labourers we able to defended their city. You killed all farmers and only ", KilledL, " labourers").
		
%	Kill, Crush & Destroy		
	robOrFight(Choice, Soldaten) :-
		Choice=2,
		peopleStatus(computer, CompF, CompL, CompS),
		fightFactorStatus(human, FightFactor),
		army(human, HumAttackers, HumDefenders),
		CompStrength=(CompF+CompL)*0.8, 
		CompStrength < Soldaten*FightFactor,
		retract(peopleStatus(computer, CompF, CompL, CompS)),
		assert(peopleStatus(computer, 0, 0, 0)),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, Soldaten, HumDefenders)),
		NewS=Soldaten+HumDefenders,
		retract(peopleStatus(human, HumF, HumL, _)),
		assert(peopleStatus(human, HumF, HumL, NewS)),
		write("Vrag zahodit v gorod, plennyh ne schadja \n").
		
%	Repeat if not sensible answer
	robOrFight(Choice, Soldaten) :-
		Choice<>1,
		Choice<>2,
		write("Pardon? Plunder or fight?"),
		readint(NewChoice),
		robOrFight(NewChoice, Soldaten).
		
	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY/FIGHT WARS/COMPUTER 
predicates
	nondeterm fightComputerWar.
	nondeterm compRobOrFight(integer remainingAttackers)
	
clauses
	%	No war
	fightComputerWar :-
		army(computer, 0, _).
	
%	Normal war, computer wins
	fightComputerWar :-
		army(human, HumAttackers, HumDefenders),
		army(computer, CompAttackers, CompDefenders),
		wallStatus(human, HumWall),
		fightFactorStatus(human, HumFightFactor),
		fightFactorStatus(computer, CompFightFactor),
		CompAttackers*CompFightFactor > (HumDefenders+HumWall)*HumFightFactor,
		NewCompAttackers=val(integer, (CompFightFactor*CompAttackers-HumFightFactor*HumDefenders)/(CompFightFactor*CompAttackers)*(1+
					(CompFightFactor*CompAttackers-HumFightFactor*HumDefenders)/(CompFightFactor*CompAttackers))*CompAttackers),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, HumAttackers, 0)),
		retract(army(computer, CompAttackers, CompDefenders)),
		assert(army(human, NewCompAttackers, CompDefenders)),
		retract(peopleStatus(computer, CompF, CompL, _)),
		NewCompS=NewCompAttackers+CompDefenders,
		assert(peopleStatus(computer, CompF, CompL, NewCompS)),
		retract(peopleStatus(human, HumF, HumL, _)),		
		assert(peopleStatus(human, HumF, HumL, HumAttackers)),
		DiedCompAttackers=CompAttackers-NewCompAttackers,
		write("Computer has defeated your forces loosing ", DiedCompAttackers, " attackers."),
		compRobOrFight(NewCompAttackers).
				
%	Normal war, human wins
	fightComputerWar :-
		army(human, HumAttackers, HumDefenders),
		army(computer, CompAttackers, CompDefenders),
		wallStatus(human, HumWall),
		fightFactorStatus(human, HumFightFactor),
		fightFactorStatus(computer, CompFightFactor),
		CompAttackers*CompFightFactor < (HumDefenders+HumWall)*HumFightFactor,
		NewHumDefenders=val(integer, (HumFightFactor*HumDefenders-CompFightFactor*CompAttackers)/(HumFightFactor*HumDefenders)*(1+
					(HumFightFactor*HumDefenders-CompFightFactor*CompAttackers)/(HumFightFactor*HumDefenders))*HumDefenders),
		retract(army(human, HumAttackers, HumDefenders)),
		assert(army(human, HumAttackers, NewHumDefenders)),
		retract(army(computer, CompAttackers, CompDefenders)),
		assert(army(computer, 0, CompDefenders)),
		retract(peopleStatus(human, HumF, HumL, _)),
		NewHumS=NewHumDefenders+HumDefenders,
		assert(peopleStatus(human, HumF, HumL, NewHumS)),
		retract(peopleStatus(computer, CompF, CompL, _)),		
		assert(peopleStatus(computer, CompF, CompL, CompDefenders)),
		KilledDefenders=HumDefenders-NewHumDefenders,
		write("Computer has been trying to break through, attackers were vanquished but you lose ", KilledDefenders, " soldiers").
		
	compRobOrFight(CompAttackers) :-
		write("Programmer stopped here").
		
	%%%%%%%%%%%%%%%%%%%%%%%%%%% PLAY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
predicates
	nondeterm play            % controls play for a year
	
clauses

	play :-
		peopleStatus(_, 0, 0, 0).
		
	play :-
		currentYear(lASTYEAR),
		write("You seem to have discovered the secret of a peaceful and prosperous life."),
		exit.
		
	play :-
		displaySituation,	% show the human what they have
		getCommands, 		% find out what human wants to do
		makeComputerDecisions,  % determine what computer wants to do
		peopleStatus(computer, F, L, S),
		Food=F*eATFACTOR+L*eATFACTOR*1.5+S*eATFACTOR*2,
		compEat(computer, Food),
		compProduce(computer), 	
		fightHumanWar,
		fightComputerWar,
		
		currentYear(Y),
		NY = Y + 1,
		retract(currentYear(Y)),
		assert(currentYear(NY)), 
		play.

	%%%%%%%%%%%%%%%%%%%%%%%%%%% TAIL MESSAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

predicates
	nondeterm tailMessage	  % tells who won and by how much
	nondeterm winner(symbol who)

clauses
	winner(human) :- peopleStatus(computer, 0, 0, 0).
	winner(computer) :- peopleStatus(human, 0, 0, 0).		

	tailMessage :-
		currentYear(Y),
		winner(W),
		write("In the year ", Y, " the ", W, " conquered the known world.\n"),
		write("When all was said and done, here's what was left."), nl, nl,
		peopleStatus(W, F, L, S),
		write("    Farmers:   ", F, "\n"),
		write("    Labourers: ", L, "\n"),
		write("    Soldiers:  ", S, "\n").
		
goal
	setupPlayers,
	play,
	tailMessage.